#
sequential.test.norm<-function(x, y=NULL, mu0, mu1,
                        delta=NULL, sigma=NULL,
                        alpha=0.05, beta=0.1, plot=TRUE){

  if(is.null(y)){
    sample <- "one" 
    kind <- "one-sided"

    if(missing(mu0)){
      mu0 <- mean(x)
    }

    if(missing(mu1)){
      if(!missing(delta)){
        mu1 <- mu0+delta
      }else{
        stop("both mu1 and delta missing!")
      }
    } else {
      delta <- mu1-mu0
    }
    
    n <- length(x)
    m <- NULL
    if(!is.null(sigma)){
      variance="known"
    } else {
      variance="unknown"
      sigma <- sqrt(var(x))
    }
  } else {
    sample <- "two"
    kind <- "one-sided"
    
    if(!missing(mu0)&!missing(mu1)){
      delta <- mu1-mu0
    }else{
      if(is.null(delta))
        delta <- mean(y)-mean(x) ### ?
    }
   
    n <- length(x)
    m <- length(y)

    if(!is.null(sigma)){
      variance="known"
    } else {
      variance="unknown"
      if(n>=2)
        sigma1q<-var(x)
      else
        sigma1q<-Inf
      if(m>=2)
        sigma2q<-var(y)
      else
        sigma2q<-Inf
      sigma<-sqrt(((n-1)*sigma1q+(m-1)*sigma2q)/(n+m-2))
    }
  }

  ## Wald bounds:
  A <- (1-beta)/alpha
  B <- beta/(1-alpha)
  theta1 <- mu1-mu0
  
  if(variance=="unknown"){
    al=1/theta1*log(B)
    au=1/theta1*log(A)
    b=1/2*theta1
    
 #   zn <- sum(x-mu0)/sqrt(1/n*sum((x-mu0)^2))
 #   vn <- n-zn^2/n
    
  } else {
    al=1/theta1*log(B)
    au=1/theta1*log(A)
    b=1/2*theta1
 #   zn <- sum(x-mu0)/sigma
 #   vn <- n
  }
  
#  if(al<zn-b*vn & zn-b*vn<au) res="continue"
#  if(zn-b*vn<al) res="H0"
#  if(zn-b*vn>au) res="H1"

  ret<-list(x=x,y=y,n=n,m=m,alpha=alpha,beta=beta,
            dist="normal", sample=sample, variance=variance,
            A=A,B=B,theta1=theta1,step=0,
            al=al,au=au,b=b,vn=NULL,zn=NULL, mu0=mu0, mu1=mu1,
            sigma=sigma, delta=delta, zn=NULL, vn=NULL,
            result="continue")
  class(ret)<-"sequential.test"
  
   ## do initial update, this fills ret$x, ret$y and ret$zn and ret$vn:
  if(ret$sample=="one"){
    for(xi in x){
#      print(xi)
      ret <- update(ret,x=xi,initial=TRUE)
      if(ret$result!="continue")
        break
    }
  } else {
    l <- min(n,m)
    ret <- update(ret,x=x[1],y[1],initial=TRUE)
    for(i in 2:l){
      ret <- update(ret,x=x[i],initial=TRUE)
      if(ret$result!="continue")
        break
      ret <- update(ret,y=y[i],initial=TRUE)
      if(ret$result!="continue")
        break
    }
    if(n<m && ret$result=="continue"){
      for(i in (l+1):m){
        ret <- update(ret,y=y[i],initial=TRUE)
        if(ret$result!="continue")
          break
      }
    }
    if(n>m && ret$result=="continue"){
      for(i in (l+1):n){
        ret <- update(ret,x=x[i],initial=TRUE)
        if(ret$result!="continue")
          break
      }
    }
  }

  if(plot){
#    plot(ret)
    print(ret)
    
  }
  ret 
}

sequential.test.bernoulli<-function(x, p0=0.1, p1=0.9, alpha=0.05, beta=0.1){

  # x is a 0-1 vector
  # TODO add a check for this
  
  n<-length(x)
  r<-sum(x) # no of successes
  
  al=log(beta/(1-alpha))/log(p1*(1-p0)/(p0*(1-p1)))
  au=log((1-beta)/alpha)/log(p1*(1-p0)/(p0*(1-p1)))
  b=log((1-p0)/(1-p1))/log(p1*(1-p0)/(p0*(1-p1)))
  zn <- r-n*p0
  vn <- n*p0*(1-p0)

  if(al<zn-b*vn & zn-b*vn<au) res="continue"
  if(zn-b*vn<al) res="H0"
  if(zn-b*vn>au) res="H1"

  ret<-list(x=x,n=n,r=r,alpha=alpha,beta=beta, dist="bernoulli",
            al=al,au=au,b=b,vn=vn,zn=zn, p0=p0, p1=p1,
            result=res)
  class(ret)<-"sequential.test"
  print(ret)
  ret
}

update.sequential.test <- function(obj,x, y=NULL, initial=FALSE,plot="last"){
  if(length(x)!=1)
    stop("add only single data per step!")
  if(!inherits(obj,"sequential.test"))
    stop("obj is not a sequential test object!")
  
  if(obj$res!="continue")
    warning("Sequential test was already finished!")

  if(length(x)>1 || length(y)>1){
    ## do recursive calls
    if(obj$sample=="one"){
      for (xi in x){
        obj <- update(obj,xi)
        if(obj$result!="continue")
          break
      }
      ret <- obj
      if(plot=="last"){
        plot(ret)
        print(ret)
      }
    } else {
      n <- length(x)
      m <- length(y)
      l <- min(n,m)
      if(l>0)
        for(i in 1:l){
          obj <- update(obj,x=x[i])
          if(obj$result!="continue")
            break
          obj <- update(obj,y=y[i])
          if(obj$result!="continue")
            break
        }
      
      if(n<m && obj$result=="continue"){
        for(i in (l+1):m){
          obj <- update(obj,y=y[i])
          if(obj$result!="continue")
            break
        }
      }
      if(n>m && obj$result=="continue"){
        for(i in (l+1):n){
          obj <- update(obj,x=x[i])
          if(obj$result!="continue")
            break
        }
      }
    }
    
    ret <- obj
    if(plot=="last"){
#      plot(ret)
      print(ret)
    }
  } else {
    ##single data given
    
    ## update data and sizes
    
    obj$step <- obj$step+1
                                        #browser()
    if(obj$sample=="two"){
      if(is.null(x) && is.null(y))
        stop("both x an y are null!")
    } else {
      if(is.null(x))
        stop("x is null!")
    }
    
    if(!is.null(x)){
      obj$x<-c(obj$x,x)
      obj$n<-obj$n+1
    }
    if(obj$sample=="two"){
      if(!is.null(y)){
        obj$y<-c(obj$y,y)
        obj$m<-obj$m+1
      } 
    }

    ## calculate vn,zn:

    if(obj$dist=="normal"){
      ret <- update.sequential.test.norm(obj,x,y,initial)
    }
                       
    if(obj$dist=="bernoulli"){
      ret <- update.sequential.test.prop(obj,x,y,initial)
    }

    ## use a,b,vn,zn for decision:
    ## default:
    ret$result="continue"

   if(is.na(ret$sigma)){
      ## continue if samples too small to estimate sigma 
      ret$result="continue"
    } else {
      if(is.na(ret$zn[ret$step]) | is.na(ret$vn[ret$step])){
        ret$result="continue"
      } else {
        ret$result <- decide.sequential.test(ret)
      }
    }
    
    
    if(!initial & plot=="all"){
      plot(ret)  
      print(ret)
    }
  } # end single / multiple data
  ret 
    
#################################
  }

decide.sequential.test <- function(obj){
  if(obj$dist=="normal"){
    if(obj$mu0<obj$mu1){
       parorder <- "ascending"
     }else{
       parorder <- "descending"
     }
   }
   if(obj$dist=="bernoulli"){
     if(obj$p0<obj$p1){
       parorder <- "ascending"
     }else{
       parorder <- "descending"
     }
     
   } 
   
   if(parorder=="ascending"){
     dir <- "down"
     upper <-  obj$au + obj$b*obj$vn[obj$step]
     lower <-  obj$al + obj$b*obj$vn[obj$step]
   } else {
     dir <- "up"
     upper <- -( obj$au + obj$b*obj$vn[obj$step])
     lower <- -( obj$al + obj$b*obj$vn[obj$step])
   }

   
   
  if(upper > obj$zn[obj$step] &
      obj$zn[obj$step] > lower)
    obj$result="continue"
  
  if((obj$zn[obj$step] <=  lower && dir=="down") |
     (obj$zn[obj$step] >=  upper && dir=="up"))
    obj$result="H1"
  
  if((obj$zn[obj$step] <=  lower && dir=="up") |
     (obj$zn[obj$step] >=  upper && dir=="down"))
    obj$result="H0"

  obj$result
}


update.sequential.test.norm <- function(obj,x=NULL, y=NULL, initial=FALSE){

  if(!inherits(obj,"sequential.test"))
    stop("obj is not a sequential test object!")
  if(!initial)
    if(obj$result!="continue")
      stop("Sequential test was already finished!")

  #
  ## updates:
  if(obj$variance=="unknown" && !initial){
    ## update unknown variance:
    if(obj$sample=="one"){
      ## one sample
      obj$sigma <- sqrt(var(obj$x))
    } else {
    ## two sample
      if(obj$n>=2)
        sigma1q<-var(obj$x)
    else
      sigma1q<-Inf
    if(obj$m>=2)
      sigma2q<-var(obj$y)
    else
      sigma2q<-Inf
    
    ## obj$sigma<-sqrt(((obj$n-1)*sigma1q+(obj$m-1)*sigma2q)/(obj$n+obj$m-2))
    obj$sigma <- sqrt((sum(obj$x^2)+sum(obj$y^2)-
                       ((sum(obj$x)+sum(obj$y))^2/(obj$n+obj$m)))/
                      (obj$n+obj$m))
    }
  }

  
  ## calculate zn, vn:
  
  if(obj$sample=="one"){
    ## one sample
    if(obj$variance=="unknown"){
      obj$zn[obj$step] <- sum(obj$x-obj$mu0)/sqrt(1/obj$n*sum((obj$x-obj$mu0)^2))
      ##sum(obj$x-obj$mu0)/sqrt(sum((obj$x-obj$mu0)^2)/obj$n)
      obj$vn[obj$step] <- obj$n-obj$zn[obj$step]^2/obj$n
      ##obj$vn[obj$step] <- obj$n-obj$zn[obj$step]^2/(2*obj$n)
    } else {
      obj$zn[obj$step] <- sum(obj$x-obj$mu0)/obj$sigma
      obj$vn[obj$step] <- obj$n
      browser()
    }
  } else {
######### ????????
    ## two sample, example 5.10, sigma unknown?
    ## mean ? mu0/mu1 ?????
    Snsqrd <- (sum(obj$x^2) + sum(obj$y^2) -
               ((sum(obj$x) + sum(obj$y))^2/(obj$n + obj$m)))/(obj$n + obj$m)
    
    if(!is.na(Snsqrd) & Snsqrd!=0){
      obj$zn[obj$step] <- obj$n*obj$m/(obj$n+obj$m)*
        (mean(obj$x)-mean(obj$y))/sqrt(Snsqrd)
      obj$vn[obj$step] <- obj$n*obj$m/(obj$n+obj$m)-obj$zn[obj$step]^2/
        (2*(obj$n+obj$m))
    } else {
      obj$zn[obj$step] <- NA
      obj$vn[obj$step] <- NA
    }
  }
  
  
  obj  
}

update.sequential.test.bernoulli <- function(obj,x=NULL, y=NULL, initial=FALSE){

  if(!inherits(obj,"sequential.test"))
    stop("obj is not a sequential test object!")
  if(!initial)
    if(obj$result!="continue")
      stop("Sequential test was already finished!")
  
}
print.sequential.test <- function(x, ...){
  if(!inherits(x,"sequential.test"))
   stop("x is not a sequential test object!")
  cat(paste("Sequential Test for",x$dist,"distribution\n\n"))
  if(x$dist=="norm"){
    if(!is.null(x$sigma))
      cat(paste("Sigma known:", x$sigma,"\n"))
    else
      cat("Sigma unknown\n")
    cat(paste("\nH0: mu=",x$mu0," versus H1: mu=",x$mu1,"\n"))
  }
  if(x$dist=="bernoulli"){
    cat(paste("\nH0: p=",x$p0," versus H1: p=",x$p1,"\n"))
  }
  cat(paste("alpha:",x$alpha," beta:", x$beta,"\n\n"))
  if(x$res=="continue"){
    cat("Test not finshed, continue by adding single data via update()\n")
    cat("\nBounds: al=",x$al,"< zn-b*vn =", x$zn-x$b*x$vn, "<", x$au,"=au\n")
  } else {
   cat(paste("Test finished: accept", x$res,"\n"))
       cat("\nBounds: zn-b*vn =", x$zn-x$b*x$vn," not in  [",x$al,",", x$au,"= [al,au]\n")
  }
}
