sim.t.test<-function(n,delta=1,sigma=1,alpha=0.05,
                     target=c("p.value","conf.int"),
                     target.ci=c("typeA","typeB"),
                     type=c("one.sample","paired","two.sample.eq","two.sample"),
                     alternative=c("two.sided", "less", "greater"),
                     sigma.type=c("estimated","known")) 
{
  
  # t-test on means
  doe.t.test<-function(n,mu,sigma.mu,alpha,target,
                       target.ci,type,alternative,sigma.type)
    {
      
      # distribution type
      if (sigma.type=="known") {
        pdist<-function(x,df,lower.tail=TRUE) pnorm(x,lower.tail=lower.tail)
        qdist<-function(x,df) qnorm(x)      
      }
      else {
        pdist<-function(x,df,lower.tail=TRUE) pt(x,df,lower.tail=lower.tail)
        qdist<-function(x,df) qt(x,df)
      }

    # calculating test statistic acording to type of t-test
      if (type%in%c("one.sample","paired")) {
        s<-sigma.mu[1]
        stat<-mu[1]/s
        df<-n[1]-1
      }
      
      if (type=="two.sample.eq") {
        df<-sum(n)-2
        s<-sqrt(sum(sigma.mu^2*(n-1)*n)/df*sum(1/n))
        stat<-(mu[1]-mu[2])/s  
      }
      
      if (type=="two.sample") {
        s<-sqrt(sum(sigma.mu^2))
        stat<-(mu[1]-mu[2])/s 
        df<-s^4/sum(sigma.mu^4/(n-1))
      }

                                        # p-value and conf.int
      if (alternative == "less") {
        if (target=="p.value") 
          return(pdist(stat, df))
        else if (target.ci=="typeA") return(s*qdist(1-alpha, df)) else return(s*(stat+qdist(1-alpha, df)))
      }
      else if (alternative == "greater") {
        if (target=="p.value") 
          return(pdist(stat, df, lower.tail = FALSE))
        else if (target.ci=="typeA")
          return(s*qdist(1-alpha, df))
        else
          return(s*(stat-qdist(1-alpha, df)))
      }
      else {
        if (target=="p.value") 
          return(2 * pdist(-abs(stat), df))
        else if (target.ci=="typeA")
          return(s*qdist(1 - alpha/2, df))
        else
          return(s*c(stat-qdist(1 - alpha/2, df),stat+qdist(1 - alpha/2, df)))
      }
    }


  type<-match.arg(type)
  alternative<-match.arg(alternative)
  target<-match.arg(target)
  target.ci<-match.arg(target.ci)
  sigma.type<-match.arg(sigma.type)

  if (sigma.type=="known")
    rdist<-function(m, s, df) rnorm(1, m, s)
  else
    rdist<-function(m, s, df) rt(1, df)*s+m

                                        # paired -> one-sample
  if ((type=="paired")&(length(sigma)==2)) 
    sigma<-sqrt(sum(sigma^2))
  
                                        # two samples -> two parameters
  if ((length(n)==1)&(type%in%c("two.sample","two.sample.eq")))
    n<-c(n,n)
  if ((length(sigma)==1)&(type%in%c("two.sample","two.sample.eq")))
    sigma<-c(sigma,sigma)

                                        # generate sample means
  sigma.mu<-sigma/sqrt(n)
  if (type%in%c("one.sample","paired")) 
    mu<-rdist(delta,sigma.mu[1],n[1]-1)
  else {
    mu.x<-rdist(delta,sigma.mu[1],n[1]-1)
    mu.y<-rdist(0,sigma.mu[2],n[2]-1)
    mu<-c(mu.x,mu.y)
  }

  doe.t.test(n,mu,sigma.mu,alpha,target,target.ci,type,alternative,sigma.type)
}

# sim.t.test(10)
