ss.p.test <- function(n=NULL,p=0.5,delta=NA,alpha=0.05,beta=0.2,
                      type=c("one.sample","two.sample"),
                      alternative=c("two.sided","less","greater"),
                      target=c("p.value","conf.int"),
                      max.iter=1000,tol=ifelse(computation=="calculate",.Machine$double.eps^0.25,0.01),
                      N.sim=1000, trace=FALSE, ceiling.n=TRUE, computation="calculate", CPS.cor=FALSE, ...) 
{

  ss.calc.p.test <- function(n=NULL,p=0.5,delta=NA,alpha=0.05,beta=0.2,
                             type=c("one.sample","two.sample"),
                             alternative=c("two.sided","less","greater"),
                             target=c("p.value","conf.int"),
                             max.iter=1000,tol=.Machine$double.eps^0.25,trace=TRUE,ceiling.n=TRUE,CPS.cor=TRUE,...) 
    { 

      alpha.p.test<-function(n,p,beta,type,alternative)
        {
          k<-ifelse(alternative=="two.sided",2,1)
          if (type=="one.sample") 
            k*pnorm((sqrt(n)*abs(diff(p))-qnorm(1-beta)*sqrt(p[1]*(1-p[1])))/(sqrt(p[2]*(1-p[2]))),lower.tail=FALSE)
          else
            k*pnorm((sqrt(n)*abs(diff(p))-qnorm(1-beta)*sqrt(sum(p*(1-p))))/sqrt(sum(p)*(1-sum(p)/2)),lower.tail=FALSE)          
        }

      type<-match.arg(type)
      alternative<-match.arg(alternative)
      target<-match.arg(target)
      note<-NULL;

      if (target=="conf.int") {
                                        # normal approximation 
        if (is.null(p)) sigma<-NULL else sigma<-sqrt(p*(1-p))

        tmp<-ss.t.test(n=n,sigma=sigma,delta=delta,
                       alpha=alpha,beta=beta,type=sub("two.sample","two.sample.eq",type),alternative=alternative,sigma.type="known",
                       target=target,max.iter=max.iter,tol=tol,trace=trace,ceiling.n=ceiling.n)
        tmp$note<-c(tmp$note,"Based on normal approximation")
        if (type=="two.sample"&length(tmp$n)==1) tmp$n<-c(tmp$n,tmp$n)
        if (!is.null(p)) tmp$sigma<-sqrt(p*(1-p))

        return( structure(list(n=tmp$n,p=(1-sqrt(1-4*tmp$sigma^2))/2,delta=tmp$delta,alpha=tmp$alpha,beta=tmp$beta,
                               type=type,alternative=tmp$alternative,target=target,
                               note=tmp$note,computation="calculated",method = tmp$method.name), class = "doe.htest"))                
        
      }
      else{
        
        if (is.null(n)) {
          method.name<-paste("Minimal sample size estimation for",sub("[.]","-",type),"probability test"); 
          f<-function(n) alpha-alpha.p.test(n,p,beta,type,alternative)               
          n<-doe.uniroot(f,5,100,max.iter=max.iter,xtype="real",trace=trace)
          if (CPS.cor&type=="two.sample") n<-(ceiling(n)/4)*(1+sqrt(1+4/ceiling(n)/abs(diff(p))))^2
          if (ceiling.n) {note<-paste("Original value",round(n,2),"rounded up to",ceiling(n)); n<-ceiling(n);}
        } 
        
        if (is.null(alpha))  {
          method.name<-paste("Type-I-error estimation for",sub("[.]","-",type),"probability test");
          alpha<-alpha.p.test(n,p,beta,type,alternative)
        }
        
        if (is.null(beta)) {
          method.name<-paste("Type-II-error estimation for",sub("[.]","-",type),"probability test")
          f<-function(beta) alpha-alpha.p.test(n,p,beta,type,alternative)   
          beta<-doe.uniroot(f,0.01,0.99,max.iter=max.iter,tol=tol,xtype="real",trace=trace)
        } 
        
        if (is.null(p)) {
          method.name<-paste("Minimal p estimation for",sub("[.]","-",type),"probability test"); 
          f<-function(p) alpha-alpha.p.test(n,p,beta,type,alternative) 
          p<-doe.uniroot(f,0.01,0.5,max.iter=max.iter,tol=tol,xtype="real",trace=trace)
        }
        
        return(structure(list(n=n,p=p,delta=delta,alpha=alpha,beta=beta,
                              type=type,alternative=alternative,
                              target=target,note=note,computation="calculated",method = method.name), class = "doe.htest"))
      }  
      
    }
  
  ss.sim.p.test <- function(n=NULL,p=0.5,delta=NA,alpha=0.05,beta=0.2,
                            type=c("one.sample","two.sample"),
                            alternative=c("two.sided","less","greater"),
                            target=c("p.value","conf.int"),
                            max.iter=1000,tol=.Machine$double.eps^0.25,N.sim=1000,trace=TRUE...) 
    {
      
      type<-match.arg(type)
      alternative<-match.arg(alternative)
      target<-match.arg(target)
      note<-NULL;
      
      if (is.null(n)) {
        method.name<-paste("Minimal sample size estimation for",sub("[.]","-",type),"probability test");
        
        if (target=="p.value") {  
          f<-function(n) if (sim.p.test(n,p,alpha,target,target.ci="typeA",type,alternative)<alpha) return(0) else return(1)
          f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
        }
        
        if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
          f<-function(n) sim.p.test(n,p,alpha,target,target.ci="typeA",type,alternative)
          f.optim<-function(x) delta-doe.sim(f,x,trhold=delta) 
        }
        
        if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
          truth<-ifelse(type=="two.sample",diff(p),p)
          f<-function(n) if (max(abs(sim.p.test(n,p,alpha,target,target.ci="typeB",type,alternative)-truth))<delta) return(0) else return(1)
          f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
        }
        
        n<-doe.uniroot(f.optim,lower=5,upper=100,max.iter=max.iter,trace=trace)
        if (type=="two.sample") n<-c(n,n)
      }
      
      if (is.null(p)) 
        stop('Search for "p" currently not implemented')
      
      
      if (is.null(delta)) {
        method.name<-paste("Minimal delta size estimation for",sub("[.]","-",type),"probability test");
        
        if (target=="p.value") 
          stop("Minimal delta size estimation not meaningful for hypothesis testing")

        if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
          f<-function(x) sim.p.test(x,p,alpha,target,target.ci="typeA",type,alternative)
        }
        
        if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
          f<-function(x) if (max(abs(sim.p.test(n,p,alpha,target,target.ci="typeB",type,alternative)))<x) return(0) else return(1)
          f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
        }
        
        if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) 
          delta<-doe.sim(f,n,N.sim=N.sim)
        else 
          delta<-doe.uniroot(f.optim,lower=0.1,upper=0.5,xtype="real",max.iter=max.iter,trace=trace)      
      }
      
      if (is.null(beta)) {
        method.name<-paste("Type-II-error estimation for",sub("[.]","-",type),"probability test");
        
        if (target=="p.value") {  
          f<-function(x) if (sim.p.test(x,p,alpha,target,target.ci="typeA",type,alternative)<alpha) return(0) else return(1)
        }
        
        if (target=="conf.int") {
          f<-function(x) if (sum(abs(sim.p.test(x,p,alpha,target,target.ci="typeA",type,alternative)))<delta) return(0) else return(1)
        }
        
        beta<-doe.sim(f,n,N.sim=N.sim)      
      }

      if (is.null(alpha)) {
        method.name<-paste("Type-I-error estimation for",sub("[.]","-",type),"probability test");
        
        if (target=="p.value") {  
          f<-function(x) if (sim.p.test(n,p,x,target,target.ci="typeA",type,alternative)<x) return(0) else return(1)
          f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
        }
        
        if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
          f<-function(x) sim.p.test(n,p,x,target,
                                    target.ci="typeA",type,alternative)
          f.optim<-function(x) delta-doe.sim(f,x,trhold=delta) 
        }
        
        if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
          truth<-ifelse(type=="two.sample",diff(p),p)
          f<-function(x)
            if (max(abs(sim.p.test(n,p,x,target,
                                                target.ci="typeA",type,
                                                alternative))-truth)<delta)
              return(0)
            else
              return(1)
          f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
        }
        
        alpha<-doe.uniroot(f.optim,lower=0.001,
                           upper=0.33/2,max.iter=max.iter,
                           trace=trace,xtype="real")
        
      }    
      
      structure(list(n=n,p=p,delta=delta,alpha=alpha,beta=beta,
                     type=type,alternative=alternative,
                     target=target,note=note,
                     computation="simulated",
                     method = method.name), class = "doe.htest")
      
    }

  type<-match.arg(type)
  alternative<-match.arg(alternative)
  target<-match.arg(target)
  if (!is.null(n)) if (type=="two.sample"&length(n)==1) n<-c(n,n)

  if (computation=="calculate") {
    return(ss.calc.p.test(n=n,p=p,delta=delta,
                          alpha=alpha,beta=beta,type=type,
                          alternative=alternative,
                          target=target,tol=tol,
                          max.iter=max.iter,trace=trace,
                          ceiling.n=ceiling.n,
                          CPS.cor=CPS.cor,...))
  } 
  else {
    return(ss.sim.p.test(n=n,p=p,delta=delta,
                         alpha=alpha,beta=beta,type=type,
                         alternative=alternative,target=target,
                         tol=tol,max.iter=max.iter,
                         N.sim=N.sim,trace=trace,...))
  }

}





