ss.t.test <- function(n=NULL,delta=1,sigma=1,alpha=0.05,beta=0.2,
                      type=c("one.sample","paired","two.sample","two.sample.eq"),
                      alternative=c("two.sided","less","greater"),
                      sigma.type=c("estimated","known"),
                      target=c("p.value","conf.int"), noise.factor=NULL,
                      max.iter=1000,tol=ifelse(computation=="calculate",
                                      .Machine$double.eps^0.25,0.01),
                      N.sim=1000, trace=FALSE, ceiling.n=TRUE, computation="calculate",...) 
{

  ss.calc.t.test <- function(n=NULL,
                             delta=1,
                             sigma=1,
                             alpha=0.05,
                             beta=0.2,
                             type=c("one.sample",
                               "paired",
                               "two.sample",
                               "two.sample.eq"),
                             alternative=c("two.sided",
                               "less",
                               "greater"),
                             sigma.type=c("estimated","known"),
                             target=c("p.value","conf.int"),
                             noise.factor=NULL,
                             max.iter=1000,
                             tol=.Machine$double.eps^0.25,
                             trace=TRUE,ceiling.n=TRUE,...)
    {
      alpha.t.test<-function(n,delta,sigma,beta,type,
                             alternative,sigma.type,target)
        {
          
          k<-switch(alternative,
                    "two.sided"=2,
                    "less"=1,
                    "greater"=1)
          l<-switch(type,
                    "one.sample"=1,
                    "paired"=1,
                    "two.sample"=2,
                    "two.sample.eq"=2)
          if (type=="two.sample")
            df<-sum(sigma^2/n)^2/sum(sigma^4/(n-1)/n^2) else df<-sum(n-1)
          
          if (sigma.type=="estimated") {
            dist<-pt; 
            L<-exp(log(2)+2*lgamma((df+1)/2)-2*lgamma(df/2)-log(df)-log(l))
            if (!is.na(beta)) B<-qt(1-beta/k^(target=="conf.int"),df) else B<-0
          }
          else {
            dist<-function(x,df) pnorm(x); 
            L<-1
            if (!is.na(beta)) B<-qnorm(1-beta/k^(target=="conf.int")) else B<-0
          }
          
          (1-dist(1/sqrt(sum(sigma^2/n)*L^(target=="conf.int"))*delta-B,df))*k  
        }
      
      type<-match.arg(type)
      alternative<-match.arg(alternative)
      sigma.type<-match.arg(sigma.type)
      target<-match.arg(target)
      note<-NULL;

      if (!is.null(noise.factor)) {
        coef<-noise.factor[1]+(1-noise.factor[1])/noise.factor[2]
        if (is.null(delta)) sigma<-sigma*coef else delta<-delta/coef
      }
      
      if (is.null(n)) {
        method.name<-paste("Minimal sample size estimation for",
                           sub("[.]","-",type),"T-test");
        
        if (substr(type,1,10)!="two.sample") 
          f<-function(n)
            alpha-alpha.t.test(n,delta,sigma,beta,type,
                               alternative,sigma.type,target) 
        else f<-function(n)
          alpha-alpha.t.test(c(n,sigma[2]/sigma[1]*n),delta,
                             sigma,beta,type,alternative,sigma.type,target) 
        
        n<-doe.uniroot(f,3,100,max.iter=max.iter,xtype="real",trace=trace)
        
        if (substr(type,1,10)=="two.sample") n<-c(n,sigma[2]/sigma[1]*n)
        if (ceiling.n) {note<-paste("Original value",round(n,2),
                                    "rounded up to",ceiling(n)); n<-ceiling(n);}
      } 
      
      if (is.null(alpha))  {
        method.name<-paste("Type-I-error estimation for",
                           sub("[.]","-",type),"T-test");
        alpha<-alpha.t.test(n,delta,sigma,beta,type,
                            alternative,sigma.type,target)
      }
      
      if (is.null(beta)) {
        method.name<-paste("Type-II-error estimation for",
                           sub("[.]","-",type),"T-test")
        f<-function(b)
          alpha-alpha.t.test(n,delta,sigma,b,type,
                             alternative,sigma.type,target)    
        beta<-doe.uniroot(f,0.01,0.99,max.iter=max.iter,
                          tol=tol,xtype="real",trace=trace)
      } 
      
      if (is.null(delta)) {
        method.name<-paste("Minimal delta estimation for",
                           sub("[.]","-",type),"T-test"); 
        f<-function(d)
          alpha-alpha.t.test(n,d,sigma,beta,type,alternative,sigma.type,target) 
        delta<-doe.uniroot(f,mean(sigma),mean(sigma),
                           max.iter=max.iter,tol=tol,xtype="real",trace=trace)
      }
      
      if (is.null(sigma)) {
        method.name<-paste("Maximal sigma estimation for",
                           sub("[.]","-",type),"T-test"); 
        
        if (type!="two.sample") 
          f<-function(s)
            alpha.t.test(n,delta,s,beta,type,
                         alternative,sigma.type,target)-alpha 
        else f<-function(s)
          alpha.t.test(n,delta,c(s,s*n[2]/n[1]),beta,type,
                       alternative,sigma.type,target)-alpha 
        
        sigma<-doe.uniroot(f,delta,delta,max.iter=max.iter,
                           tol=tol,xtype="real",trace=trace)
        if (type=="two.sample") sigma<-c(sigma,n[2]/n[1]*sigma)
      }
      
      
      structure(list(n=n,delta=delta,sigma=sigma,alpha=alpha,beta=beta,
                     type=type,alternative=alternative,sigma.type=sigma.type,
                     target=target,note=note,computation="calculated",
                     method = method.name), class = "doe.htest")
      
    }
  
  ss.sim.t.test <- function(n=NULL,
                            delta=1,
                            sigma=1,
                            alpha=0.05,
                            beta=0.2,
                            type=c("one.sample",
                              "paired",
                              "two.sample",
                              "two.sample.eq"),
                            alternative=c("two.sided",
                              "less",
                              "greater"),
                            sigma.type=c("estimated","known"),
                            target=c("p.value","conf.int"),
                            max.iter=1000,tol=0.01,
                            N.sim=1000,trace=TRUE,...) {
    
    type<-match.arg(type)
    alternative<-match.arg(alternative)
    sigma.type<-match.arg(sigma.type)
    target<-match.arg(target)
    note<-NULL;

    if (!is.null(noise.factor))
      stop('simulation for noise factor not implemented')
    
    if (is.null(n)) {
      method.name<-paste("Minimal sample size estimation for",
                         sub("[.]","-",type),"T-test");
      
      if (target=="p.value") {  
        f<-function(x)
          if (sim.t.test(x,delta,sigma,alpha,target,type,
                         target.ci="typeA",alternative,
                         sigma.type)<alpha)
            return(0)
          else
            return(1)
        f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
      }
      
      if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          sim.t.test(x,0,sigma,alpha,target,type,
                     target.ci="typeA",alternative,sigma.type)
        f.optim<-function(x)
          delta-doe.sim(f,x,trhold=delta) 
      }
      
      if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          if (max(abs(sim.t.test(x,0,sigma,alpha,target,
                                 type,target.ci="typeB",
                                 alternative,sigma.type)))<delta)
            return(0)
          else
            return(1)
        f.optim<-function(x)
          beta-doe.sim(f,x,trhold=beta)
      }
      if (type=="two.sample") {
        g<-f
        f<-function(x)
          g(c(x,ceiling(sigma[2]/sigma[1]*x)))}
      
      n<-doe.uniroot(f.optim,lower=3,upper=100,
                     max.iter=max.iter,trace=trace)
      if (type=="two.sample") n<-c(n,ceiling(sigma[2]/sigma[1]*n))
      
    }
    
    if (is.null(delta)) {
      method.name<-paste("Minimal delta size estimation for",
                         sub("[.]","-",type),"T-test");
      
      if (target=="p.value") {  
        f<-function(x)
          if (sim.t.test(n,x,sigma,alpha,target,
                         type,target.ci="typeA",
                         alternative,sigma.type)<alpha)
            return(0)
          else
            return(1)
        f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
      }

      if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          sim.t.test(x,0,sigma,alpha,target,
                     type,target.ci="typeA",alternative,sigma.type)
      }
      
      if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          if (max(abs(sim.t.test(n,0,sigma,alpha,
                                 target,type,target.ci="typeB",
                                 alternative,sigma.type)))<x)
            return(0)
          else
            return(1)
        f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
      }
      
      if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) 
        delta<-doe.sim(f,n,N.sim=N.sim)
      else 
        delta<-doe.uniroot(f.optim,lower=sigma,
                           upper=sigma,xtype="real",
                           max.iter=max.iter,trace=trace)
      
    }
    
    if (is.null(sigma)) {
      method.name<-paste("Maximal sigma estimation for",
                         sub("[.]","-",type),"T-test");
      
      if (target=="p.value") {  
        f<-function(x) if (sim.t.test(n,delta,x,alpha,target,
                                      type,target.ci="typeA",
                                      alternative,sigma.type)<alpha)
          return(0)
        else
          return(1)
        f.optim<-function(x) doe.sim(f,x,trhold=beta)-beta
      }
      
      if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          sim.t.test(n,0,x,alpha,target,type,
                     target.ci="typeA",alternative,sigma.type)
        f.optim<-function(x) delta-doe.sim(f,x,trhold=delta) 
      }
      
      if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          if (max(abs(sim.t.test(n,0,x,alpha,target,
                                 target.ci="typeB",type,
                                 alternative,sigma.type)))<delta)
            return(0)
          else
            return(1)
        f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
      }
      if (type=="two.sample") {
        g<-f
        f<-function(x)
          g(c(x,ceiling(n[2]/n[1]*x)))
      }
      
      sigma<-doe.uniroot(f.optim,lower=delta,upper=delta,
                         max.iter=max.iter,trace=trace,xtype="real")
      if (type=="two.sample") sigma<-c(n,ceiling(n[2]/n[1]*sigma))
      
    }
    
    if (is.null(beta)) {
      method.name<-paste("Type-II-error estimation for",
                         sub("[.]","-",type),"T-test");
      
      if (target=="p.value") {  
        f<-function(x)
          if (sim.t.test(x,delta,sigma,alpha,target,
                         type,target.ci="typeA",
                         alternative,sigma.type)<alpha)
            return(0)
          else
            return(1)
      }
      
      if (target=="conf.int") {
        f<-function(x)
          if (sum(abs(sim.t.test(x,0,sigma,alpha,
                                 target,type,target.ci="typeA",
                                 alternative,sigma.type)))<delta)
            return(0)
          else
            return(1)
      }
      
      beta<-doe.sim(f,n,N.sim=N.sim)      
    }

    if (is.null(alpha)) {
      method.name<-paste("Type-I-error estimation for",
                         sub("[.]","-",type),"T-test");
      
      if (target=="p.value") {  
        f<-function(x) if (sim.t.test(n,delta,sigma,x,
                                      target,type,target.ci="typeA",
                                      alternative,sigma.type)<x)
          return(0)
        else
          return(1)
        f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
      }
      
      if (target=="conf.int"&is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          sim.t.test(n,0,sigma,x,target,type,
                     target.ci="typeA",alternative,sigma.type)
        f.optim<-function(x) delta-doe.sim(f,x,trhold=delta) 
      }
      
      if (target=="conf.int"&!is.na(ifelse(is.null(beta),1,beta))) {
        f<-function(x)
          if (max(abs(sim.t.test(n,0,sigma,x,
                                 target,type,target.ci="typeB",
                                 alternative,sigma.type)))<delta)
            return(0)
          else
            return(1)
        f.optim<-function(x) beta-doe.sim(f,x,trhold=beta)
      }
      
      alpha<-doe.uniroot(f.optim,lower=0.001,
                         upper=0.33/2,max.iter=max.iter,
                         trace=trace,xtype="real")
      
    }    
    
    structure(list(n=n,delta=delta,sigma=sigma,alpha=alpha,beta=beta,
                   type=type,alternative=alternative,sigma.type=sigma.type,
                   target=target,note=note,computation="simulated",
                   method = method.name), class = "doe.htest")
    
  }


  type<-match.arg(type)
  alternative<-match.arg(alternative)
  sigma.type<-match.arg(sigma.type)
  target<-match.arg(target)

  if (!is.null(n)) if (type %in% c("two.sample","two.sample.eq") &
                       length(n)==1)
    n<-c(n,n)
  if (type=="two.sample.eq" & length(sigma)==2) if (!is.null(n))
    sigma<-sqrt((n-1)*sigma^2/(sum(n)-2)) else sigma<-sqrt(sum(sigma^2)/2)
  if (!is.null(sigma))
    if (type %in% c("two.sample","two.sample.eq") & length(sigma)==1)
      sigma<-c(sigma,sigma)
  if (!is.null(sigma))
    if (type == "paired")
      sigma<-sqrt(sum(sigma^2))

  if (computation=="calculate") {
    return(ss.calc.t.test(n=n,delta=delta,sigma=sigma,
                          alpha=alpha,beta=beta,type=type,
                          noise.factor=noise.factor,
                          alternative=alternative,sigma.type=sigma.type,
                          target=target,tol=tol,max.iter=max.iter,
                          trace=trace,ceiling.n=ceiling.n,...))
  } 
  else {
    return(ss.sim.t.test(n=n,delta=delta,sigma=sigma,
                         alpha=alpha,beta=beta,type=type,
                         noise.factor=noise.factor,
                         alternative=alternative,sigma.type=sigma.type,
                         target=target,tol=tol,max.iter=max.iter,
                         N.sim=N.sim,trace=trace,...))
  }

}
